# where are the Max/MSP SDK header files?
# you should have the latest version!
MAXSDKPATH="%ProgramFiles%\MaxMSP 4.5\maxmspsdk_win\4.5 headers\c74support"	

# where is MS VC++?
# (not necessary if the build is run with the compiler environment)
# MSVCPATH="%ProgramFiles%\Microsoft Visual Studio .NET 2003\Vc7"

###############################################################

# where do/should the flext headers reside/be built?
FLEXTINC="%CommonProgramFiles%\Cycling '74\flext"

# where do/should the flext static libraries reside/be built?
FLEXTLIB=$(FLEXTINC)

# where do/should the flext shared libraries reside/be built?
# (a good place is the MaxMSP program folder)
FLEXTSHLIB="%ProgramFiles%\MaxMSP 4.5"

###############################################################

# where should the external be built?
# (path for temporary files)
OUTPATH=max-msvc

# where should the external be installed?
INSTPATH="%CommonProgramFiles%\Cycling '74\externals\flext"

# where should the initialization files be installed?
INITPATH="%CommonProgramFiles%\Cycling '74\init"

# where should the help files be installed?
HELPPATH="%ProgramFiles%\MaxMSP 4.5\max-help\flext"

###############################################################

# where is the STK installation?
# (uncomment the line with # to disable STK support)
# --------------------------------------------------------------------
# In order to use STK you must have an stk.lib library
# Using MSVC you can build it as following (from the STK main folder):
#   cl /c /GR /EHsc /Iinclude /MT /Ox src\*.cpp
#     (in case you define DYNAMIC=1 below, omit the /MT flag and set /MD instead)
#     (you will get errors for some files, you can ignore them in the flext context)
#   lib *.obj /out:stk.lib
#

# where to find the STK header files (e.g. stk.h)
#STK_INC=%HOMEDRIVE%\%HOMEPATH%\stk\include

# where to find the STK library (normally stk.lib)
# (comment out STK_LIB if you don't use STK)
#STK_LIB=%HOMEDRIVE%\%HOMEPATH%\stk\stk.lib

# where is the Sndobjs installation?
# (uncomment the line with # to disable SNDOBJ support)
# --------------------------------------------------------------------
# Please be sure to delete an eventual m_pd.h file from the SNDOBJ include folder!!
# Flext must be compiled with the same compiler version as the SNDOBJ lib (default MSVC6)
#

# where to find the SndObj header files (e.g. SndObj.h)
#SNDOBJ_INC=%HOMEDRIVE%\%HOMEPATH%\SndObj\include

# where to find the SndObj library (normally SndObj.lib)
# (comment out SNDOBJ_LIB if you don't use SndObj)
#SNDOBJ_LIB=%HOMEDRIVE%\%HOMEPATH%\SndObj\lib\SndObj.lib

###############################################################

# some user-definable flags
# (check if they match your system!)
OFLAGS=/Ox
# optimizations for Pentium 4
#OFLAGS=$(OFLAGS) /G6 /arch:SSE

# uncomment to link against dynamic C runtime libraries
# (that's ok if you compile with VC++ 7.0 and 7.1)
DYNAMIC=1
