ifdef SIMD
DEFS += -DFLEXT_USE_SIMD
endif

ifdef ATOMICOPSPATH
DEFS += -DUSE_ATOMIC_OPS
endif

ifdef SNDOBJ_LIB
SRCS+=$(SRCS_SNDOBJ)
HDRS+=$(HDRS_SNDOBJ)
endif

ifdef STK_LIB
SRCS+=$(SRCS_STK)
HDRS+=$(HDRS_STK)
endif
