build-default-default: build-all-all

build-all-default: build-all-all

build-release-default: build-release-all

build-debug-default: build-debug-all

build-profile-default: build-profile-all

build-default-all: build-all-all

build-default-single: build-all-single

build-default-multi: build-all-multi

build-default-shared: build-all-shared


install-default-default: install-all-all

install-all-default: install-all-all

install-release-default: install-release-all

install-debug-default: install-debug-all

install-default-all: install-all-all

install-default-single: install-all-single

install-default-multi: install-all-multi

install-default-shared: install-all-shared


clean-default-default: clean-all-all

clean-all-default: clean-all-all

clean-release-default: clean-release-all

clean-debug-default: clean-debug-all

clean-default-all: clean-all-all

clean-default-single: clean-all-single

clean-default-multi: clean-all-multi

clean-default-shared: clean-all-shared

